/*
libfawk - A function-only AWK dialect

Copyright (c) 2017..2019 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#include "libfawk.h"
#include "execute.h"
#include <stdio.h>

FAWK_API const char *fawk_ins_name(fawk_ins_t i)
{
	switch(i) {
		case FAWKI_MAKE_SYMREF:     return "MAKE_SYMREF";
		case FAWKI_PUSH_NUM:        return "PUSH_NUM";
		case FAWKI_PUSH_STR:        return "PUSH_STR";
		case FAWKI_PUSH_SYMVAL:     return "PUSH_SYMVAL";
		case FAWKI_PUSH_TOPVAR:     return "PUSH_TOPVAR";
		case FAWKI_PUSH_REL:        return "PUSH_REL";
		case FAWKI_PUSH_NIL:        return "PUSH_NIL";
		case FAWKI_POP:             return "POP";
		case FAWKI_POPJZ:           return "POPJZ";
		case FAWKI_POPJNZ:          return "POPJNZ";
		case FAWKI_NEG:             return "NEG";
		case FAWKI_NOT:             return "NOT";
		case FAWKI_EQ:              return "EQ";
		case FAWKI_NEQ:             return "NEQ";
		case FAWKI_LTEQ:            return "LTEQ";
		case FAWKI_GTEQ:            return "GTEQ";
		case FAWKI_LT:              return "LT";
		case FAWKI_GT:              return "GT";
		case FAWKI_IN:              return "IN";
		case FAWKI_ADD:             return "ADD";
		case FAWKI_SUB:             return "SUB";
		case FAWKI_MUL:             return "MUL";
		case FAWKI_DIV:             return "DIV";
		case FAWKI_MOD:             return "MOD";
		case FAWKI_CONCAT:          return "CONCAT";
		case FAWKI_FORIN_FIRST:     return "FORIN_FIRST";
		case FAWKI_FORIN_NEXT:      return "FORIN_NEXT";
		case FAWKI_INCDEC:          return "INCDEC";
		case FAWKI_SET:             return "SET";
		case FAWKI_CALL:            return "CALL";
		case FAWKI_RET:             return "RET";
		case FAWKI_JMP:             return "JMP";
		case FAWKI_ABORT:           return "ABORT";
	}
	return "<unknown instruction>";
}

FAWK_API void fawk_dump_symref(fawk_symref_t *sr, int follow)
{
	if (sr->is_local)
		printf("{local at sp=%d}", sr->ref.local);
	else
		printf("{global %s (%p)}", sr->ref.global->name, (void *)sr->ref.global);
}


FAWK_API void fawk_dump(fawk_ctx_t *ctx)
{
	size_t n;
	for(n = 0; n < ctx->code.used; n++) {
		fawk_code_t *i = &ctx->code.code[n];
		printf("[%04ld %03d]  ", (long)n, i->line);
		switch(i->type) {
			case FAWKC_INS:    printf("INS  %s", fawk_ins_name(i->data.ins)); break;
			case FAWKC_SYMREF: printf("SYMREF "); fawk_dump_symref(i->data.symref, 0); break;
			case FAWKC_NUM:    printf("NUM  %f", i->data.num); break;
			case FAWKC_STR:    printf("STR  '%s' (ref=%ld, len=%ld/%ld)", i->data.str->str, (long)i->data.str->refco, (long)i->data.str->used, (long)i->data.str->alloced); break;
			case FAWKC_CSTR:   printf("CSTR '%s'", i->data.cstr); break;
		}
		printf("\n");
	}
}

FAWK_API void fawk_dump_stack(fawk_ctx_t *ctx)
{
	size_t n = ctx->sp;
	for(;;) {
		printf("[%ld] ", (long)n);
		fawk_dump_cell(fawk_peek(ctx, n), 1);
		printf("\n");
		if (n == 0)
			break;
		n--;
	}
	fflush(stdout);
}

